@@ -1,6 +1,6 @@
-/* $Id: BallView.h,v 1.6 2001/01/31 17:13:18 nan Exp $ */
+/* $Id: BallView.h,v 1.8 2003/07/13 15:26:24 nan Exp $ */
 
-// Copyright (C) 2000  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000, 2002, 2003  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,7 +32,11 @@ public:
 
   static GLuint m_number[10];
 private:
+  void BallView::DrawTargetCircle();
   GLUquadricObj *m_quad;
+
+  void DrawBall();
+  void DrawShadow();
 };
 
 #endif	// _BallView
