@@ -1,6 +1,6 @@
-/* $Id: Control.cpp,v 1.3 2001/09/19 14:05:22 nan Exp $ */
+/* $Id: Control.cpp,v 1.4 2002/11/04 14:13:18 nan Exp $ */
 
-// Copyright (C) 2000  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000, 2002  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,8 +18,18 @@
 
 #include "ttinc.h"
 #include "Control.h"
+#include "Player.h"
+#include "Network.h"
+#include "PlayGame.h"
+
+extern long wins;
+
+extern long score1;
+extern long score2;
 
 Control *Control::m_theControl = NULL;
+Player* Control::m_thePlayer = NULL;
+Player* Control::m_comPlayer = NULL;
 
 Control::Control() {
 }
@@ -27,3 +37,25 @@ Control::Control() {
 Control::~Control() {
   m_theControl = NULL;
 }
+
+void
+Control::ClearControl() {
+  if ( m_thePlayer && wins == 0 ) {
+    delete m_thePlayer;
+    m_thePlayer = NULL;
+    ClearSocket();
+  }
+  if ( m_comPlayer ) {
+    delete m_comPlayer;
+    m_comPlayer = NULL;
+  }
+
+  if ( m_theControl ) {
+    if ( m_theControl->IsPlaying() ) {
+      score1 = ((PlayGame *)m_theControl)->GetScore(1);
+      score2 = ((PlayGame *)m_theControl)->GetScore(-1);
+    }
+    delete m_theControl;
+    m_theControl = NULL;
+  }
+}
