@@ -1,6 +1,6 @@
-/* $Id: Control.h,v 1.4 2001/09/19 14:05:22 nan Exp $ */
+/* $Id: Control.h,v 1.6 2003/07/16 16:03:38 nan Exp $ */
 
-// Copyright (C) 2000, 2001  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000, 2001, 2002  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,13 +20,14 @@
 #define _Control_
 
 class View;
+class Player;
 
 class Control {
 public:
   virtual ~Control();
 
   virtual bool Init() = 0;
-  virtual bool Move( unsigned long *KeyHistory, long *MouseXHistory,
+  virtual bool Move( SDL_keysym *KeyHistory, long *MouseXHistory,
 		     long *MouseYHistory, unsigned long *MouseBHistory,
 		     int Histptr ) = 0;
   virtual bool LookAt( double &srcX, double &srcY, double &srcZ,
@@ -36,10 +37,17 @@ public:
 
   virtual View *GetView() = 0;
 
+  static void ClearControl();
+
   static Control *TheControl() { return m_theControl; };
+  static Player* GetThePlayer() { return m_thePlayer; };
+  static Player* GetComPlayer() { return m_comPlayer; };
 protected:
   Control();
   static Control *m_theControl;
+
+  static Player* m_thePlayer;
+  static Player* m_comPlayer;
 };
 
 #endif	// _Control_
