@@ -1,6 +1,6 @@
-/* $Id: Event.cpp,v 1.73 2002/02/14 16:46:26 nan Exp $ */
+/* $Id: Event.cpp,v 1.84 2003/11/19 17:26:58 nan Exp $ */
 
-// Copyright (C) 2000, 2001, 2002  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000-2003   ȹ(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +24,9 @@
 #include "SoloPlay.h"
 #include "PracticePlay.h"
 #include "MultiPlay.h"
+#include "Network.h"
 #include "PlayerSelect.h"
+#include "MultiPlayerSelect.h"
 #include "Title.h"
 #include "Opening.h"
 #include "Howto.h"
@@ -40,12 +42,10 @@
 extern RCFile *theRC;
 
 extern Ball theBall;
-extern Player* thePlayer;
-extern Player* comPlayer;
 extern long mode;
 
 extern void Timer( int value );
-struct timeb Event::m_lastTime = {0, 0, 0, 0};	// $BD>A0$K(BTimerEvent$B$,8F$P$l$?$H$-$N;~9o(B
+struct timeb Event::m_lastTime = {0, 0, 0, 0};	// ľTimerEventƤФ줿Ȥλ
 
 Event* Event::m_theEvent = NULL;
 
@@ -55,8 +55,8 @@ extern long wins;
 
 extern bool isComm;
 
-extern long score1;
-extern long score2;
+extern SDL_mutex *networkMutex;
+extern long timeAdj;
 
 long _perfCount;
 long perfs;
@@ -69,7 +69,7 @@ void CopyPlayerData( Player& dest, Player* src );
 void QuitGame();
 
 Event::Event() {
-  m_KeyHistory[0] = 0;
+  memset( &(m_KeyHistory[0]), 0, sizeof(SDL_keysym) );
   m_MouseXHistory[0] = BaseView::GetWinWidth()/2;
   m_MouseYHistory[0] = BaseView::GetWinHeight()/2;
   m_MouseBHistory[0] = 0;
@@ -92,6 +92,9 @@ Event::TheEvent() {
 
 bool
 Event::Init() {
+  if (isComm)
+    m_External = new ExternalNullData();
+
   switch ( mode ){
   case MODE_SOLOPLAY:
     SoloPlay::Create( 0, RAND(2) );
@@ -99,9 +102,15 @@ Event::Init() {
   case MODE_MULTIPLAY:
     MultiPlay::Create( 0, RAND(2) );
     break;
+  case MODE_HOWTO:
+    Howto::Create();
+    break;
   case MODE_SELECT:
     PlayerSelect::Create();
     break;
+  case MODE_MULTIPLAYSELECT:
+    MultiPlayerSelect::Create();
+    break;
   case MODE_TITLE:
     Title::Create();
     break;
@@ -112,9 +121,6 @@ Event::Init() {
 
   theBall.Init();
 
-  if (isComm)
-    m_External = new ExternalNullData();
-
   return true;
 }
 
@@ -161,6 +167,10 @@ Event::IdleFunc() {
   perfs +=diffcount;
   _perfCount++;
 
+#ifdef SCREENSHOT
+  diffcount = 3;
+#endif
+
   for ( int i = 0 ; i < diffcount ; i++ ) {
     // While pause, never move objects. (Solo Play)
     if ( Control::TheControl()->IsPlaying() &&
@@ -182,6 +192,8 @@ Event::IdleFunc() {
     long preMode = mode;
     Event::TheEvent()->ReadData();
     Event::TheEvent()->IsModeChanged( preMode );
+  } else if ( mode == MODE_MULTIPLAYSELECT ) {
+    Event::TheEvent()->ReadSelectData();
   }
 
   if ( mode != MODE_OPENING && mode != MODE_TITLE &&
@@ -211,7 +223,7 @@ Event::Move() {
 
 bool
 Event::IsModeChanged( long preMode ) {
-  if ( mode != preMode ){	// $B%b!<%IJQ99$"$j(B
+  if ( mode != preMode ) {	// mode change
     long p, q;
 
     switch ( mode ) {
@@ -223,11 +235,15 @@ Event::IsModeChanged( long preMode ) {
       break;
     case MODE_MULTIPLAY:
       p = ((PlayerSelect *)Control::TheControl())->GetPlayerNum();
-      MultiPlay::Create( p, 0 );
+      q = ((PlayerSelect *)Control::TheControl())->GetOpponentNum();
+      MultiPlay::Create( p, q );
       break;
     case MODE_SELECT:
       PlayerSelect::Create();
       break;
+    case MODE_MULTIPLAYSELECT:
+      MultiPlayerSelect::Create();
+      break;
     case MODE_TITLE:
       Title::Create();
       break;
@@ -265,7 +281,7 @@ Event::IsModeChanged( long preMode ) {
 void
 Event::Record() {
   long x, y;
-  long key;
+  SDL_keysym key;
   long btn;
 
   x = m_MouseXHistory[m_Histptr];
@@ -284,7 +300,8 @@ Event::Record() {
   char buf[1024];
 
   if ( mode == MODE_MULTIPLAY )
-    cnt += ((MultiPlay *)Control::TheControl())->GetTimeAdj();
+    cnt += timeAdj;
+
   while ( cnt < 0 ) {
     sec--;
     cnt += 100;
@@ -296,7 +313,7 @@ Event::Record() {
 
   sprintf( buf, "%d.%2d %d - %d  x = %4.2f y = %4.2f z = %4.2f st = %d\n", 
 	   sec, cnt,
-	   m_BacktrackBuffer[m_Histptr].score1,
+	   m_BacktrackBuffer[m_Histptr].score1, 
 	   m_BacktrackBuffer[m_Histptr].score2, 
 	   m_BacktrackBuffer[m_Histptr].theBall.GetX(), 
 	   m_BacktrackBuffer[m_Histptr].theBall.GetY(),
@@ -382,7 +399,7 @@ Event::Record() {
   // Warp Mouse Pointer
   SetNextMousePointer( x, y );
 
-  m_KeyHistory[m_Histptr] = 0;
+  memset( &(m_KeyHistory[m_Histptr]), 0, sizeof(SDL_keysym) );
   m_MouseXHistory[m_Histptr] = x;
   m_MouseYHistory[m_Histptr] = y;
   m_MouseBHistory[m_Histptr] = btn;
@@ -391,8 +408,10 @@ Event::Record() {
   m_BacktrackBuffer[m_Histptr].count = m_lastTime.millitm/10;
 
   m_BacktrackBuffer[m_Histptr].theBall = theBall;
-  CopyPlayerData( m_BacktrackBuffer[m_Histptr].thePlayer, thePlayer );
-  CopyPlayerData( m_BacktrackBuffer[m_Histptr].comPlayer, comPlayer );
+  CopyPlayerData( m_BacktrackBuffer[m_Histptr].thePlayer,
+		  Control::GetThePlayer() );
+  CopyPlayerData( m_BacktrackBuffer[m_Histptr].comPlayer,
+		  Control::GetComPlayer() );
 
   if ( mode == MODE_SOLOPLAY || mode == MODE_MULTIPLAY ||
        mode == MODE_PRACTICE ) {
@@ -444,12 +463,12 @@ Event::KeyboardFunc( SDL_Event key, int x, int y ) {
     }
   }
 
-  Event::TheEvent()->m_KeyHistory[Event::TheEvent()->m_Histptr] = key.key.keysym.unicode;
+  Event::TheEvent()->m_KeyHistory[Event::TheEvent()->m_Histptr] = key.key.keysym;
 }
 
 void
 Event::KeyUpFunc( SDL_Event key, int x, int y ) {
-  // $BGQ;_$7$F$b$h$$(B
+  // ѻߤƤ褤
 #if 0
   if ( key.key.keysym.unicode == 'Q' ) {
     QuitGame();
@@ -538,28 +557,6 @@ Event::GetExternalData( ExternalData *&ext, long side ) {
   return true;
 }
 
-bool
-Event::SendSwing( Player *player ) {
-  char buf[256];
-
-  if ( m_backtrack || mode != MODE_MULTIPLAY )
-    return false;
-
-  strncpy( buf, "PS", 2 );
-  ((MultiPlay *)Control::TheControl())->SendTime( &(buf[2]) );
-
-  player->SendSwing( &(buf[7]) );
-
-  // Player $B0LCV>pJs$bAw?.$9$k(B
-  strncpy( &(buf[31]), "PV", 2 );
-  ((MultiPlay *)Control::TheControl())->SendTime( &(buf[33]) );
-
-  player->SendLocation( &(buf[38]) );
-
-  send( theSocket, buf, 31+55, 0 );
-  return true;
-}
-
 bool
 Event::SendPlayer( Player *player ) {
   char buf[256];
@@ -616,35 +613,11 @@ Event::SendPlayerAndBall( Player *player ) {
   return true;
 }
 
-void
-Event::ClearObject() {
-  if ( thePlayer && wins == 0 ) {
-    delete thePlayer;
-    thePlayer = NULL;
-    if ( theSocket >= 0 ) {
-      send( theSocket, "QT", 2, 0 );
-      closesocket( theSocket );
-      theSocket = -1;
-    }
-  }
-  if ( comPlayer ) {
-    delete comPlayer;
-    comPlayer = NULL;
-  }
-  if ( Control::TheControl() ) {
-    if ( Control::TheControl()->IsPlaying() ) {
-      score1 = ((PlayGame *)Control::TheControl())->GetScore(1);
-      score2 = ((PlayGame *)Control::TheControl())->GetScore(-1);
-    }
-    delete Control::TheControl();
-  }
-}
-
 bool
 Event::BackTrack( long Histptr ) {
   theBall = m_BacktrackBuffer[Histptr].theBall;
-  thePlayer->Reset( &m_BacktrackBuffer[Histptr].thePlayer );
-  comPlayer->Reset( &m_BacktrackBuffer[Histptr].comPlayer );
+  Control::GetThePlayer()->Reset( &m_BacktrackBuffer[Histptr].thePlayer );
+  Control::GetComPlayer()->Reset( &m_BacktrackBuffer[Histptr].comPlayer );
 
   if ( mode == MODE_SOLOPLAY || mode == MODE_MULTIPLAY ||
        mode == MODE_PRACTICE) {
@@ -662,24 +635,15 @@ Event::ReadData() {
   fd_set rdfds;
   struct timeval to;
 
-  while (1) {
-    FD_ZERO( &rdfds );
-    FD_SET( (unsigned int)theSocket, &rdfds );
+  // Caution!! exchanged with WaitForData()
 
-    to.tv_sec = to.tv_usec = 0;
+  SDL_mutexP( networkMutex );
 
-    if ( select( theSocket+1, &rdfds, NULL, NULL, &to ) > 0 ) {
-      if ( !GetExternalData( m_External, comPlayer->GetSide() ) )
-	break;
-    } else
-      break;
-  }
-
-  // externalData$B$N@hF,$^$G(Bbacktrack$B$9$k(B
+  // externalDataƬޤbacktrack. 
   long btCount = 0;
   ExternalData *externalOld;
   long btHistptr;
-  while ( !(m_External->isNull()) ) {	// $B8E$9$.$k>pJs$r<N$F$k(B
+  while ( !(m_External->isNull()) ) {	// ŤΤƤ. 
     btCount = (m_lastTime.time-m_External->sec)*100 + 
       (m_lastTime.millitm/10-m_External->count);
     if ( btCount > MAX_HISTORY ) {
@@ -712,26 +676,27 @@ Event::ReadData() {
 
     BackTrack( btHistptr );
 
-    // $BE,MQ$9$k(B -> $B?J$a$k$r(BbtCount$B7+$jJV$9(B
+    // ŬѤ -> ʤbtCount֤
     while (1) {
       if ( fTheBall )
 	theBall.Move();
 
       if ( fThePlayer )
-	thePlayer->Move( m_KeyHistory, m_MouseXHistory,
-			 m_MouseYHistory, m_MouseBHistory, m_Histptr );
+	Control::GetThePlayer()->Move( m_KeyHistory, m_MouseXHistory,
+				       m_MouseYHistory, m_MouseBHistory,
+				       m_Histptr );
 
       if ( fComPlayer )
-	comPlayer->Move( NULL, NULL, NULL, NULL, 0 );
+	Control::GetComPlayer()->Move( NULL, NULL, NULL, NULL, 0 );
 
       while ( !(m_External->isNull()) &&
 	      m_External->sec == m_lastTime.time &&
 	      m_External->count == m_lastTime.millitm/10 ) {
 	Player *targetPlayer;
-	if ( m_External->side == thePlayer->GetSide() )
-	  targetPlayer = thePlayer;
-	else if ( m_External->side == comPlayer->GetSide() )
-	  targetPlayer = comPlayer;
+	if ( m_External->side == Control::GetThePlayer()->GetSide() )
+	  targetPlayer = Control::GetThePlayer();
+	else if ( m_External->side == Control::GetComPlayer()->GetSide() )
+	  targetPlayer = Control::GetComPlayer();
 	else {
 	  xerror("%s(%d) ExternalData", __FILE__, __LINE__);
 	  exit(1);
@@ -759,9 +724,11 @@ Event::ReadData() {
 	}
       }
       if ( fThePlayer )
-	CopyPlayerData( m_BacktrackBuffer[m_Histptr].thePlayer, thePlayer );
+	CopyPlayerData( m_BacktrackBuffer[m_Histptr].thePlayer,
+			Control::GetThePlayer() );
       if ( fComPlayer )
-	CopyPlayerData( m_BacktrackBuffer[m_Histptr].comPlayer, comPlayer );
+	CopyPlayerData( m_BacktrackBuffer[m_Histptr].comPlayer, 
+			Control::GetComPlayer() );
 
       m_lastTime.millitm += 10;
       if ( m_lastTime.millitm >= 1000 ) {
@@ -781,23 +748,52 @@ Event::ReadData() {
 		     m_BacktrackBuffer[m_Histptr].score2 );
     }
     if (!fThePlayer)
-      thePlayer->Reset( &m_BacktrackBuffer[m_Histptr].thePlayer );
+      Control::GetThePlayer()->Reset(&m_BacktrackBuffer[m_Histptr].thePlayer);
     if (!fComPlayer)
-      comPlayer->Reset( &m_BacktrackBuffer[m_Histptr].comPlayer );
+      Control::GetComPlayer()->Reset(&m_BacktrackBuffer[m_Histptr].comPlayer);
   }
+
+  SDL_mutexV( networkMutex );
+}
+
+void
+Event::ReadSelectData() {
+  fd_set rdfds;
+  struct timeval to;
+  bool dum;
+  ExternalData *externalOld;
+
+  SDL_mutexP( networkMutex );
+
+  while ( !(m_External->isNull()) ) {
+    if ( m_External->dataType != DATA_PT )  // Selection is already finished
+      break;
+
+    m_External->Apply( NULL, dum, dum, dum );
+
+    externalOld = m_External;
+    m_External = m_External->next;
+    delete externalOld;
+  }
+
+  SDL_mutexV( networkMutex );
 }
 
 void
 Event::ClearBacktrack() {
+  Ball aBall;
+
   for ( int i = 0 ; i < MAX_HISTORY ; i++ ) {
     Event::TheEvent()->m_MouseXHistory[i] = 0;
     Event::TheEvent()->m_MouseYHistory[i] = 0;
     Event::TheEvent()->m_MouseBHistory[i] = 0;
     Event::TheEvent()->m_BacktrackBuffer[i].sec = m_lastTime.time;
     Event::TheEvent()->m_BacktrackBuffer[i].count = m_lastTime.millitm/10;
-    Event::TheEvent()->m_BacktrackBuffer[i].theBall = theBall;
-    CopyPlayerData( Event::TheEvent()->m_BacktrackBuffer[i].thePlayer, thePlayer );
-    CopyPlayerData( Event::TheEvent()->m_BacktrackBuffer[i].comPlayer, comPlayer );
+    Event::TheEvent()->m_BacktrackBuffer[i].theBall = aBall;
+    CopyPlayerData( Event::TheEvent()->m_BacktrackBuffer[i].thePlayer, 
+		    Control::GetThePlayer() );
+    CopyPlayerData( Event::TheEvent()->m_BacktrackBuffer[i].comPlayer,
+		    Control::GetComPlayer() );
 
     Event::TheEvent()->m_BacktrackBuffer[i].score1 = 0;
     Event::TheEvent()->m_BacktrackBuffer[i].score2 = 0;
@@ -831,7 +827,7 @@ Event::SetNextMousePointer( long &x, long &y ) {
   if ( mode == MODE_TITLE ) {
     return;
   } else if ( mode == MODE_SELECT || mode == MODE_TRAININGSELECT ||
-	      mode == MODE_PRACTICESELECT ) {
+	      mode == MODE_PRACTICESELECT || mode == MODE_MULTIPLAYSELECT ) {
     x = BaseView::GetWinWidth()/2 + (x-BaseView::GetWinWidth()/2)*15/16;
   } else {
     x = BaseView::GetWinWidth()/2 + (x-BaseView::GetWinWidth()/2)*15/16;
@@ -843,7 +839,8 @@ Event::SetNextMousePointer( long &x, long &y ) {
 void
 Event::GetAdjustedTime( long &sec, long &cnt ) {
   if ( isComm )
-    cnt += ((MultiPlay *)Control::TheControl())->GetTimeAdj();
+    //cnt += ((MultiPlay *)Control::TheControl())->GetTimeAdj();
+    cnt += timeAdj;
   while ( cnt < 0 ) {
     sec--;
     cnt += 100;
@@ -867,7 +864,8 @@ Event::RemainingLog() {
     char buf[1024];
 
     if ( isComm )
-      cnt += ((MultiPlay *)Control::TheControl())->GetTimeAdj();
+      //cnt += ((MultiPlay *)Control::TheControl())->GetTimeAdj();
+      cnt += timeAdj;
     while ( cnt < 0 ) {
       sec--;
       cnt += 100;
@@ -889,3 +887,8 @@ Event::RemainingLog() {
 }
 
 #endif
+
+bool
+Event::GetExternalData( long side ) {
+  return GetExternalData( m_External, side );
+}
