@@ -1,6 +1,6 @@
-/* $Id: Network.h,v 1.1 2001/12/05 14:06:54 nan Exp $ */
+/* $Id: Network.h,v 1.9 2003/01/19 15:41:00 nan Exp $ */
 
-// Copyright (C) 2001  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2001-2003  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,15 +33,33 @@ char *ReadDouble( char *buf, double& d );
 char *ReadLong( char *buf, long& l );
 
 // Send/Recv data
+void ReadHeader( int socket, char *buf );
+long ReadEntireMessage( int socket, char **buf );
+
 void WritePlayerData();
-Player *ReadPlayerData();
-void SendPlayerData();
+
+void SendSwing( Player *player );
 
 void SendTime( int sd, struct timeb* tb );
 void ReadTime( int sd, struct timeb* tb );
 
 void getcurrenttime( struct timeb *tb );
 void ReadBI();
+#ifdef ENABLE_IPV6
+struct addrinfo* findhostname();
+#else
 void findhostname( struct sockaddr_in *saddr );
+#endif
+
+void ClearSocket();
+bool GetSocket();
+
+bool WaitForClient();
+bool ConnectToServer();
+void ServerAdjustClock();
+void ClientAdjustClock();
+
+// For exception. 
+struct NetworkError { };
 
 #endif	// _Network_
