@@ -1,6 +1,6 @@
-/* $Id: Player.cpp,v 1.41 2002/02/19 16:13:05 nan Exp $ */
+/* $Id: Player.cpp,v 1.55 2003/11/19 16:49:31 nan Exp $ */
 
-// Copyright (C) 2000, 2001  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000-2003   ȹ(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,9 +44,6 @@ extern RCFile *theRC;
 
 extern Ball   theBall;
 
-extern Player *thePlayer;
-extern Player *comPlayer;
-
 extern long mode;
 
 Player::Player() {
@@ -129,9 +126,6 @@ Player::Player( long side ) {
   m_lastSendX = m_lastSendY = m_lastSendZ = 0.0;
   m_lastSendVX = m_lastSendVY = m_lastSendVZ = 0.0;
   m_lastSendCount = 0;
-  m_lastSendX = m_lastSendY = m_lastSendZ = 0.0;
-  m_lastSendVX = m_lastSendVY = m_lastSendVZ = 0.0;
-  m_lastSendCount = 0;
 }
 
 Player::Player( long playerType, long side, double x, double y, double z, 
@@ -275,10 +269,7 @@ Player::Create( long player, long side, long type ) {
 
 bool
 Player::Init() {
-  if ( theRC->gmode == GMODE_2D )
-    m_View = new PlayerView2D();
-  else
-    m_View = new PlayerView();
+  m_View = (PlayerView *)View::CreateView( VIEW_PLAYER );
 
   m_View->Init( this );
 
@@ -298,7 +289,7 @@ Player::Reset( Player *p ) {
 }
 
 bool
-Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
+Player::Move( SDL_keysym *KeyHistory, long *MouseXHistory,
 	      long *MouseYHistory, unsigned long *MouseBHistory,
 	      int Histptr ) {
   //static double  lastSendX = 0,  lastSendY = 0,  lastSendZ = 0;
@@ -309,10 +300,6 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
   if ( m_swing > 0 ){
     if ( m_swing > 30 && m_afterSwing > 0 ) {
       m_afterSwing--;
-      /*
-      if ( m_afterSwing%10 == 0 )
-	m_swing++;
-	*/
     } else {
       if ( theBall.GetStatus() == 6 || theBall.GetStatus() == 7 ) {
 	if ( theBall.GetVZ() < 0 )
@@ -405,7 +392,8 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
   if ( m_swing == 20 ){
     HitBall();
 
-    if ( thePlayer == this && theRC->gmode != GMODE_2D ) {
+    if ( Control::TheControl()->GetThePlayer() == this &&
+	 theRC->gmode != GMODE_2D ) {
       HitMark *hit;
 
       hit = new HitMark();
@@ -439,20 +427,20 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
 
       if ( ((tmpBall->GetStatus() == 3 && m_side == 1) ||
 	    (tmpBall->GetStatus() == 1 && m_side == -1)) ) {
-	double xdiff = tmpBall->GetX() - (m_x+m_vx*(20-m_swing)*TICK);
+	double hitpointX = m_swingSide ? m_x+0.3*m_side : m_x-0.3*m_side;
+	double xdiff = tmpBall->GetX() - (hitpointX+m_vx*(20-m_swing)*TICK);
 	double ydiff = tmpBall->GetY() - (m_y+m_vy*(20-m_swing)*TICK);
 
 	double vxdiff, vydiff;
-	if ( xdiff > 0.0 )
-	  vxdiff = (xdiff-0.3)/TICK/(20-m_swing);
-	else
-	  vxdiff = (xdiff+0.3)/TICK/(20-m_swing);
+	vxdiff = xdiff/TICK/(20-m_swing);
 
 	if ( vxdiff > 2.0 )
 	  vxdiff = 2.0;
 	else if ( vxdiff < -2.0 )
 	  vxdiff = -2.0;
 
+	vxdiff /= theRC->gameLevel+1;
+
 	m_vx += vxdiff;
 
 	if ( fabs(ydiff) > 0.3 ) {
@@ -461,6 +449,9 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
 	    vydiff = 2.0;
 	  else if ( vydiff < -2.0 )
 	    vydiff = -2.0;
+
+	  vydiff /= theRC->gameLevel+1;
+
 	  m_vy += vydiff;
 	}
       }
@@ -507,7 +498,6 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
   } else
     m_y += m_vy*TICK;
 
-#if 0
 // Go back to the endline before serve
   if ( Control::TheControl()->IsPlaying() && theBall.GetStatus() == 8 &&
        ((PlayGame *)Control::TheControl())->GetService() == GetSide() ) {
@@ -516,7 +506,6 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
     else if ( m_side < 0 && m_y < TABLELENGTH/2 )
       m_y = TABLELENGTH/2;
   }
-#endif
 
   // Auto backswing
   if ( m_swing == 0 ) {
@@ -544,7 +533,7 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
   if ( hypot( m_vx, m_vy ) > 2.0 )
     AddStatus( -1 );
 
-  if ( m_swing > 0 )
+  if ( m_swing > 10 )
     AddStatus( -1 );
 
   if ( theBall.GetStatus() == 8 || theBall.GetStatus() == -1 )
@@ -553,7 +542,8 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
   if ( SDL_WM_GrabInput( SDL_GRAB_QUERY ) == SDL_GRAB_ON )
     KeyCheck( KeyHistory, MouseXHistory, MouseYHistory, MouseBHistory,Histptr );
 
-  if ( thePlayer == this && mode == MODE_MULTIPLAY ) {
+  if ( Control::TheControl()->GetThePlayer() == this &&
+       mode == MODE_MULTIPLAY ) {
     m_lastSendCount++;
 
     m_lastSendX += m_lastSendVX*TICK;
@@ -579,17 +569,107 @@ Player::Move( unsigned long *KeyHistory, long *MouseXHistory,
 }
 
 bool
-Player::KeyCheck( unsigned long *KeyHistory, long *MouseXHistory,
+Player::KeyCheck( SDL_keysym *KeyHistory, long *MouseXHistory,
 		  long *MouseYHistory, unsigned long *MouseBHistory,
 		  int Histptr ) {
   long mouse, lastmouse;
 
+const char keytable[][5] = {
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'1', '1', '1', '[', '\0'},
+  {'2', '2', '2', '7', '\0'},
+  {'3', '3', '3', '5', '\0'},
+  {'4', '4', '4', '3', '\0'},
+  {'5', '5', '5', '1', '\0'},
+  {'6', '6', '6', '9', '\0'},
+  {'7', '7', '7', '0', '\0'},
+  {'8', '8', '8', '2', '\0'},
+  {'9', '9', '9', '6', '\0'},
+  {'0', '0', '0', '8', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'q', 'q', 'a', '/', '\0'},
+  {'w', 'w', 'z', ',', '\0'},
+  {'e', 'e', 'e', '.', '\0'},
+  {'r', 'r', 'r', 'p', '\0'},
+  {'t', 't', 't', 'y', '\0'},
+  {'y', 'z', 'y', 'f', '\0'},
+  {'u', 'u', 'u', 'g', '\0'},
+  {'i', 'i', 'i', 'c', '\0'},
+  {'o', 'o', 'o', 'r', '\0'},
+  {'p', 'p', 'p', 'l', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'a', 'a', 'q', 'a', '\0'},
+  {'s', 's', 's', 'o', '\0'},
+  {'d', 'd', 'd', 'e', '\0'},
+  {'f', 'f', 'f', 'u', '\0'},
+  {'g', 'g', 'g', 'i', '\0'},
+  {'h', 'h', 'h', 'd', '\0'},
+  {'j', 'j', 'j', 'h', '\0'},
+  {'k', 'k', 'k', 't', '\0'},
+  {'l', 'l', 'l', 'n', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},	// {          'm', 's', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},
+  {'z', 'y', 'w', ';', '\0'},
+  {'x', 'x', 'x', 'q', '\0'},
+  {'c', 'c', 'c', 'j', '\0'},
+  {'v', 'v', 'v', 'k', '\0'},
+  {'b', 'b', 'b', 'x', '\0'},
+  {'n', 'n', 'n', 'b', '\0'},
+  {'m', 'm'     , 'm', '\0', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},	// {               'w', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'},	// {               'v', '\0'},
+  {'\0', '\0', '\0', '\0', '\0'}	// {               'z', '\0'}
+};
+
 // COM
   if ( !KeyHistory || !MouseXHistory || !MouseYHistory || !MouseBHistory )
     return true;
 
 // key input
-  switch ( KeyHistory[Histptr] ) {
+  // Check keyboard type and modify keycode. 
+  int code = -1;
+
+  if ( KeyHistory[Histptr].scancode < 54 ) {
+    int i = 0;
+    while (keytable[KeyHistory[Histptr].scancode][i]) {
+      if ( keytable[KeyHistory[Histptr].scancode][i]
+	   == KeyHistory[Histptr].unicode ) {
+	code = keytable[KeyHistory[Histptr].scancode][0];
+	break;
+      }
+      i++;
+    }
+  }
+
+  if ( KeyHistory[Histptr].scancode >= 8 && 
+       KeyHistory[Histptr].scancode < 62 && 
+       code < 0 ) {	// for X11
+    int i = 0;
+    while (keytable[KeyHistory[Histptr].scancode-8][i]) {
+      if ( keytable[KeyHistory[Histptr].scancode-8][i]
+	   == KeyHistory[Histptr].unicode ) {
+	code = keytable[KeyHistory[Histptr].scancode-8][0];
+	break;
+      }
+      i++;
+    }
+  }
+
+  if ( code < 0 )
+    code = KeyHistory[Histptr].unicode;
+
+
+  switch ( code ) {
   case '1':  case 'q':  case 'a':  case 'z':
   case '2':  case 'w':  case 's':  case 'x':
   case '3':
@@ -636,7 +716,7 @@ Player::KeyCheck( unsigned long *KeyHistory, long *MouseXHistory,
     break;
   }
 
-  switch ( KeyHistory[Histptr] ){
+  switch ( code ){
   case '1':  case '2':  case '3':  case '4':  case '5':  case '6':
   case '7':  case '8':  case '9':  case '0':  case '-':  case '^':
     m_targetY = TABLELENGTH/12*5*GetSide();
@@ -655,9 +735,12 @@ Player::KeyCheck( unsigned long *KeyHistory, long *MouseXHistory,
     break;
   }
 
-  if ( (Histptr == 0 && KeyHistory[Histptr] != KeyHistory[MAX_HISTORY-1]) ||
-       (Histptr != 0 && KeyHistory[Histptr] != KeyHistory[Histptr-1]) ) {
-    switch ( KeyHistory[Histptr] ){
+
+  if ( (Histptr == 0 &&
+	KeyHistory[Histptr].unicode != KeyHistory[MAX_HISTORY-1].unicode) ||
+       (Histptr != 0 &&
+	KeyHistory[Histptr].unicode != KeyHistory[Histptr-1].unicode) ) {
+    switch ( KeyHistory[Histptr].unicode ) {
     case 'H':
       m_eyeX -= 0.05;
       break;
@@ -676,19 +759,39 @@ Player::KeyCheck( unsigned long *KeyHistory, long *MouseXHistory,
     case '>':
       m_eyeY += 0.05;
       break;
+
+    case 'A':
+      m_lookAtX -= 0.05;
+      break;
+    case 'S':
+      m_lookAtZ -= 0.05;
+      break;
+    case 'D':
+      m_lookAtZ += 0.05;
+      break;
+    case 'F':
+      m_lookAtX += 0.05;
+      break;
+    case 'C':
+      m_lookAtY -= 0.05;
+      break;
+    case 'V':
+      m_lookAtY += 0.05;
+      break;
+
     }
   }
 
   // Sorry in Japanese...
-  // $B%9%$%s%0Cf$OB.EY$,JQ$o$i$J$$$h$&$K$9$k(B. $B$3$&$9$k$3$H$G(B
-  // MultiPlay $B;~$K$h$jF14|$r$H$j$d$9$/$J$k(B. 
-  // $B$=$NM}M3$O(B, $B%9%$%s%0$r3+;O$7$?;~E@$G(B Player $B$N%$%s%Q%/%H(B
-  // $B$^$G$N9TF0$,7hDj$5$l$k$?$a(B, $B%9%$%s%03+;O;~$K>pJs8r49$9$l$P(B
-  // $BF14|$,$H$l$k$h$&$K$J$k$?$a$G$"$k(B. $B$3$l$^$G$O(B, $B%$%s%Q%/%H;~$K(B
-  // $B>pJs$r8r49$7$F$$$?$N$G(B, 0.1$BICAa$/F14|$,$H$l$k$3$H$K$J$k(B. 
+  // ®٤Ѥʤ褦ˤ. 뤳Ȥ
+  // MultiPlay ˤƱȤ䤹ʤ. 
+  // ͳ, 󥰤򳫻Ϥ Player Υѥ
+  // ޤǤιưꤵ뤿, 󥰳ϻ˾򴹤
+  // ƱȤ褦ˤʤ뤿Ǥ. ޤǤ, ѥȻ
+  // 򴹤ƤΤ, 0.1᤯ƱȤ뤳Ȥˤʤ. 
 
-  // $B$A$J$_$K(B, $B%9%$%s%08e$N%^%&%9$N%I%i%C%0$K$h$C$F%\!<%k$N2sE>$r(B
-  // $B@)8f$9$k>l9g(B, $B$3$NJ}K!$O;H$($J$$$+$bCN$l$J$$(B. 
+  // ʤߤ, 󥰸ΥޥΥɥåˤäƥܡβž
+  // 椹, ˡϻȤʤΤʤ. 
 
   if ( SDL_WM_GrabInput( SDL_GRAB_QUERY ) == SDL_GRAB_OFF )
     return true;
@@ -746,16 +849,6 @@ Player::KeyCheck( unsigned long *KeyHistory, long *MouseXHistory,
     }
   }
 
-#if 0
-  if ( mouse & (BUTTON_RIGHT|BUTTON_MIDDLE|BUTTON_LEFT) &&
-       m_swing > 10 && m_swing <= 20 ) {
-    m_pow++;
-    if ( m_pow > 10 )
-      m_pow = 10;
-  }
-#else
-#endif
-
   return true;
 }
 
@@ -776,23 +869,23 @@ Player::AddStatus( long diff ) {
     }
 
     if ( diff < -3 ) {	// Not good... When status decreased without moving...
-      if ( this == thePlayer ) {
+      if ( Control::TheControl()->GetThePlayer() == this ) {
 	switch (theRC->gameLevel) {
 	case LEVEL_EASY:
-	  m_statusMax = (m_statusMax*3+m_status)/4;
+	  m_statusMax += diff/4;
 	  break;
 	case LEVEL_NORMAL:
-	  m_statusMax = (m_statusMax*2+m_status)/3;
+	  m_statusMax += diff/3;
 	  break;
 	case LEVEL_HARD:
-	  m_statusMax = (m_statusMax+m_status)/2;
+	  m_statusMax += diff/2;
 	  break;
 	case LEVEL_TSUBORISH:
-	  m_statusMax = (m_statusMax+m_status)/2;
+	  m_statusMax += diff/2;
 	  break;
 	}
       } else {
-	m_statusMax = (m_statusMax*3+m_status)/4;	/* $BBG5e0LCV$G$N%Z%J%k%F%#$rAj;&(B */
+	m_statusMax += diff/4;	/* ǵ֤ǤΥڥʥƥ껦 */
       }
     }
   }
@@ -1365,3 +1458,49 @@ Player::UpdateLastSend() {
   m_lastSendVY = m_vy;
   m_lastSendVZ = m_vz;
 }
+
+void
+Player::AddError( double &vx, double &vy, double &vz ) {
+  double v;
+  double n1x, n1y, n1z, n2x, n2y, n2z;
+  double radDiff, radRand;
+
+#if 0
+  radDiff = hypot( fabs(fabs(m_x-theBall.GetX())-0.3)/0.3, 
+		   fabs(m_y-theBall.GetY())/0.3 );
+  radDiff = sqrt( radDiff );
+  radDiff *= (double)(200-m_status)/200*3.141592/12;
+#else
+  radDiff = (double)(200-m_status)/200*3.141592/12;
+#endif
+
+  v = sqrt(vx*vx+vy*vy+vz*vz);
+  n1x = vy/hypot(vx, vy) * v*tan(radDiff);
+  n1y = -vx/hypot(vx, vy) * v*tan(radDiff);
+  n1z = 0;
+  n2x = vx*vz/(v*hypot(vx, vy)) * v*tan(radDiff);
+  n2y = vy*vz/(v*hypot(vx, vy)) * v*tan(radDiff);
+  n2z = (vx*vx+vy*vy)/(v*hypot(vx, vy)) * v*tan(radDiff);
+
+  // Hit the ball too fast --- net miss
+  // Hit the ball too slow --- over miss
+  if ( (m_y-theBall.GetY())*m_side < 0 )
+    radRand = (RAND(180)+180)*3.141592/180.0;
+  else
+    radRand = RAND(180)*3.141592/180.0;
+
+  vx += n1x*cos(radRand)+n2x*sin(radRand);
+  vy += n1y*cos(radRand)+n2y*sin(radRand);
+  vz += n1z*cos(radRand)+n2z*sin(radRand);
+}
+
+// If status point is less than this value, player will miss. 
+long
+Player::StatusBorder() {
+  double nearEdge = TABLEWIDTH/2-fabs(m_targetX);
+  if ( TABLELENGTH/4-fabs(fabs(m_targetY)-TABLELENGTH/4) < nearEdge )
+    nearEdge = TABLELENGTH/4-fabs(fabs(m_targetY)-TABLELENGTH/4);
+
+  return 50+(TABLELENGTH/4-nearEdge)*40;
+
+}
