@@ -1,6 +1,6 @@
-/* $Id: RCFile.h,v 1.3 2001/12/05 14:41:00 nan Exp $ */
+/* $Id: RCFile.h,v 1.5 2003/07/28 16:56:15 nan Exp $ */
 
-// Copyright (C) 2001  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2001, 2003   ȹ(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,12 +26,13 @@ public:
   static RCFile *GetRCFile();
 
   bool isTexture;
-  bool isWireFrame;
   bool fullScreen;
+  long myModel;
   long gmode;
   long gameLevel;
   long gameMode;
   long sndMode;
+  long protocol;
 
   char serverName[256];
   char nickname[32];
