@@ -1,4 +1,4 @@
-/* $Id: TrainingSelect.h,v 1.7 2001/12/26 14:24:20 nan Exp $ */
+/* $Id: TrainingSelect.h,v 1.8 2003/07/16 15:55:24 nan Exp $ */
 
 // Copyright (C) 2000, 2001  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
@@ -30,7 +30,7 @@ public:
 
   static void Create();
 
-  virtual bool Move( unsigned long *KeyHistory, long *MouseXHistory,
+  virtual bool Move( SDL_keysym *KeyHistory, long *MouseXHistory,
 		     long *MouseYHistory, unsigned long *MouseBHistory,
 		     int Histptr );
 
