@@ -1,6 +1,6 @@
-/* $Id: TrainingView.h,v 1.2 2001/01/07 17:48:38 nan Exp $ */
+/* $Id: TrainingView.h,v 1.3 2002/11/04 14:37:00 nan Exp $ */
 
-// Copyright (C) 2000  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
+// Copyright (C) 2000, 2002  $B?@Fn(B $B5H9((B(Kanna Yoshihiro)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,22 +18,20 @@
 
 #ifndef _TrainingView_
 #define _TrainingView_
-#include "View.h"
+#include "PlayGameView.h"
 
 class Training;
 
-class TrainingView : public View {
+class TrainingView : public PlayGameView {
 public:
   TrainingView();
   virtual ~TrainingView();
 
-  bool Init( Training * );
+  virtual bool Init( PlayGame * );
 
-  virtual bool Redraw();
   virtual bool RedrawAlpha();
 
 protected:
-  Training     *m_training;
   GLuint        m_max;
 };
 
