@@ -1,6 +1,6 @@
 /***********************************************************************
  *	z:/wata/src/a/csmash/loadparts.cpp
- *	$Id: loadparts.cpp,v 1.12 2002/03/05 14:21:21 yotsuya Exp $
+ *	$Id: loadparts.cpp,v 1.14 2003/11/19 16:49:31 nan Exp $
  *
  *	Copyright by ESESoft.
  *
@@ -430,7 +430,7 @@ bool parts::load_anim(int lineno, anim_parts* object,
 	}
     }
     if (!i) {
-	printf("%d: %s is empty object\n", lineno, object->name);
+	printf("%d: %s is empty object\n", lineno, object->name.c_str());
     }
     return true;
 }
@@ -500,15 +500,7 @@ bool texture_parts::realize()
     ImageData img;
     bool loaded;
 
-#if !defined(CHIYO) || !defined(WIN32)
     loaded = img.LoadFile(filename.c_str());
-#else
-    if (LANG_HINDI == PRIMARYLANGID(GetUserDefaultLangID())) {
-	loaded = img.LoadFile("images/ntb.jpg");
-    } else {
-	loaded = img.LoadFile(filename.c_str());
-    }
-#endif
 
     if (!loaded) {
         throw verror("could not load texture %s\n", filename.c_str());
